@@ -13,6 +13,7 @@ from bentoutils.kubeutil import from_yaml, delete_from_yaml
 from bentoutils.print_kaniko_manifest import camel_to_kebab, gen_manifest as gen_kaniko_manifest, gen_docker_manifest
 from bentoutils.print_knative_manifest import gen_manifest as gen_knative_manifest
 from bentoutils.print_route_manifest import gen_manifest as gen_route_manifest
+from bentoutils.print_bento_manifest import gen_manifest as gen_bento_manifest
 
 #yatai_proto = LazyLoader('yatai_proto', globals(), 'bentoml.yatai.proto')
 
@@ -213,6 +214,39 @@ def get_kaniko_manifest(bento, path, registry, s3_endpoint, insecure=False, isdo
     return manifest, pod_name
 
 
+@click.command()
+@click.option('--bento', help="bento name in 'name:version' format")
+@click.option('--download_url', help="path to saved bento tar file. If not specified, yatai-image-builder will fetch this information from yatai.")
+@click.option('--selector', help="comma separated list of key=val pairs", default=None)
+@click.option('--svcname', help="service name", default=None)
+@click.option('--tolerations', help='tolerations', default=None)
+@click.option('--labels', help='comma separated list of key:val pairs', default=None)
+@click.option('--uid', help="unique ID")
+def get_bento_manifest(bento, download_url=None, selector=None, svcname=None, tolerations=None, labels=None, uid=None, silent=False):
+    
+    if ':' in bento:
+        name, version = bento.split(':')
+    else:
+        name = bento
+        version = 'latest'
+
+    if uid is None:
+        uid = str(uuid.uuid4())[:5]
+
+    if selector is not None:
+        selector = convert_labels(selector)
+
+    if labels is not None:
+        labels = convert_labels(labels)
+
+    bento_tag = '{}:{}'.format(name, version)
+    image_name = camel_to_kebab(name)
+    pod_name = '{}-{}-{}'.format(image_name, version, uid)
+    manifest = gen_bento_manifest(bento_tag, pod_name, download_url, selector, svcname, tolerations, labels)
+    if not silent: click.echo(manifest)
+    return manifest, pod_name
+
+
 @click.command()
 @click.option('--bento', help="bento name in 'name:version' format")
 @click.option('--registry', help="private registry in 'host:port' format")
@@ -324,6 +358,37 @@ def delete_containerize_job(ctx, bento, pod, registry, isdockerconfig=False):
     delete_from_yaml(manifest)
 
 
+@click.command()
+@click.option('--bento', help="bento name in 'name:version' format", required=True)
+@click.option('--download_url', help="path to saved bento tar file. If not specified, yatai-image-builder will fetch this information from yatai.", default=None)
+@click.option('--selector', help="comma separated list of key=val pairs", default=None)
+@click.option('--svcname', help="service name", default=None)
+@click.option('--tolerations', help='tolerations', default=None)
+@click.option('--labels', help='comma separated list of key:val pairs', default=None)
+@click.pass_context
+def apply_bento_crd(ctx, bento, download_url=None, selector=None, svcname=None, tolerations=None, labels=None, uid=None):
+    manifest, pod_name = ctx.invoke(get_bento_manifest, bento=bento, download_url=download_url, selector=selector, svcname=svcname, tolerations=tolerations, labels=labels, uid=uid, silent=True)
+    
+    from_yaml(manifest)
+    click.echo(pod_name)
+    return pod_name
+
+
+@click.command()
+@click.option('--bento', help="bento name in 'name:version' format", required=True)
+@click.option('--pod', help="pod name", required=True)
+@click.option('--download_url', help="path to saved bento tar file. If not specified, yatai-image-builder will fetch this information from yatai.", default=None)
+@click.option('--selector', help="comma separated list of key=val pairs", default=None)
+@click.option('--svcname', help="service name", default=None)
+@click.option('--tolerations', help='tolerations', default=None)
+@click.option('--labels', help='comma separated list of key:val pairs', default=None)
+@click.pass_context
+def delete_bento_crd(ctx, bento, pod, download_url=None, selector=None, svcname=None, tolerations=None, labels=None, uid=None):
+    uid = pod.split('-')[-1]
+    manifest, _ = ctx.invoke(get_bento_manifest, bento=bento, download_url=download_url, selector=selector, svcname=svcname, tolerations=tolerations, labels=labels, uid=uid, silent=True)
+    delete_from_yaml(manifest)
+
+
 @click.command()
 @click.option('--bento', help='bento service name')
 @click.option('--registry', help='registry name')
